@@ -1,4 +1,5 @@
 import importlib
+from typing import List, Type
 
 import h5py
 import ipyvolume as ipv
@@ -10,32 +11,41 @@ import pandas as pd
 import pythreejs
 from betagen import betagen
 from IPython.display import Markdown, Math, display
-from nptyping import NDArray
 
 from popsynth.utils.array_to_cmap import array_to_cmap
 from popsynth.utils.hdf5_utils import (
-    clean_graph_dict,
-    fill_graph_dict,
+    clean_graph_dict, fill_graph_dict,
     recursively_load_dict_contents_from_group,
-    recursively_save_dict_contents_to_group,
-)
+    recursively_save_dict_contents_to_group)
 from popsynth.utils.spherical_geometry import xyz
 
+
+from popsynth.utils.logging import setup_logger
+
+
+log = setup_logger(__name__)
+
+#from numpy.typing import ArrayLike
+
+# dummy
+ArrayLike = List[float]
+
 wine = "#8F2727"
-dark, dark_highlight, mid, mid_highlight, light, light_highlight = betagen(wine)
+dark, dark_highlight, mid, mid_highlight, light, light_highlight = betagen(
+    wine)
 
 
 class Population(object):
     def __init__(
         self,
-        luminosities: NDArray[np.float64],
-        distances: NDArray[np.float64],
-        known_distances: NDArray[np.float64],
-        known_distance_idx: NDArray[np.float64],
-        unknown_distance_idx: NDArray[np.float64],
-        fluxes: NDArray[np.float64],
-        flux_obs: NDArray[np.float64],
-        selection: NDArray[bool],
+        luminosities: ArrayLike,
+        distances: ArrayLike,
+        known_distances: ArrayLike,
+        known_distance_idx: ArrayLike,
+        unknown_distance_idx: ArrayLike,
+        fluxes: ArrayLike,
+        flux_obs: ArrayLike,
+        selection: ArrayLike,
         flux_sigma: float,
         r_max: float,
         boundary: float,
@@ -87,26 +97,25 @@ class Population(object):
         :rtype:
 
         """
-        self._luminosities = luminosities  # type: NDArray[np.float64]
+        self._luminosities = luminosities  # type: ArrayLike
 
-        self._distances = distances  # type: NDArray[np.float64]
-        self._known_distances = known_distances  # type: NDArray[np.float64]
-        self._known_distance_idx = known_distance_idx  # type: NDArray[bool]
-        self._unknown_distance_idx = unknown_distance_idx  # type: NDArray[bool]
+        self._distances = distances  # type: ArrayLike
+        self._known_distances = known_distances  # type: ArrayLike
+        self._known_distance_idx = known_distance_idx  # type: ArrayLike
+        self._unknown_distance_idx = unknown_distance_idx  # type: ArrayLike
 
-        self._theta = theta  # type: NDArray[np.float64]
-        self._phi = phi  # type: NDArray[np.float64]
+        self._theta = theta  # type: ArrayLike
+        self._phi = phi  # type: ArrayLike
 
         assert len(known_distances) + len(unknown_distance_idx) == sum(
-            selection
-        ), "the distances are not the correct size"
+            selection), "the distances are not the correct size"
 
         # latent fluxes
-        self._fluxes = fluxes  # type: NDArray[np.float64]
+        self._fluxes = fluxes  # type: ArrayLike
 
         # observed fluxes
-        self._flux_obs = flux_obs  # type: NDArray[np.float64]
-        self._selection = selection  # type: NDArrayfloat]
+        self._flux_obs = flux_obs  # type: ArrayLike
+        self._selection = selection  # type: ArrayLike
         self._flux_sigma = flux_sigma  # type: float
 
         self._r_max = r_max  # type: float
@@ -119,13 +128,13 @@ class Population(object):
         self._spatial_form = spatial_form
         self._lf_form = lf_form
 
-        self._flux_selected = flux_obs[selection]  # type: NDArray[np.float64]
-        self._distance_selected = distances[selection]  # type: NDArray[np.float64]
-        self._luminosity_selected = luminosities[selection]  # type: NDArray[np.float64]
+        self._flux_selected = flux_obs[selection]  # type: ArrayLike
+        self._distance_selected = distances[selection]  # type: ArrayLike
+        self._luminosity_selected = luminosities[selection]  # type: ArrayLike
 
-        self._flux_hidden = flux_obs[~selection]  # type: NDArray[np.float64]
-        self._distance_hidden = distances[~selection]  # type: NDArray[np.float64]
-        self._luminosity_hidden = luminosities[~selection]  # type: NDArray[np.float64]
+        self._flux_hidden = flux_obs[~selection]  # type: ArrayLike
+        self._distance_hidden = distances[~selection]  # type: ArrayLike
+        self._luminosity_hidden = luminosities[~selection]  # type: ArrayLike
 
         self._lf_params = lf_params
         self._spatial_params = spatial_params
@@ -147,7 +156,7 @@ class Population(object):
 
             self._no_detection = True
 
-            print("THERE ARE NO DETECTED OBJECTS IN THE POPULATION")
+            log.warning("THERE ARE NO DETECTED OBJECTS IN THE POPULATION")
 
         else:
 
@@ -161,7 +170,7 @@ class Population(object):
                 setattr(self, "%s_obs" % k, v["obs_values"])
                 setattr(self, "%s_selected" % k, v["obs_values"][selection])
 
-        self._auxiliary_quantites = auxiliary_quantities
+        self._auxiliary_quantities = auxiliary_quantities
 
         if model_spaces is not None:
 
@@ -210,15 +219,25 @@ class Population(object):
             return self._distance_probability
 
     @property
-    def theta(self) -> NDArray[np.float64]:
+    def theta(self) -> np.ndarray:
         return self._theta
 
     @property
-    def phi(self) -> NDArray[np.float64]:
+    def phi(self) -> np.ndarray:
         return self._phi
 
     @property
-    def luminosities_latent(self) -> NDArray[np.float64]:
+    def dec(self) -> np.ndarray:
+        return 90 - np.rad2deg(self._theta)
+
+    @property
+    def ra(self) -> np.ndarray:
+        return np.rad2deg(self._phi)
+
+
+    
+    @property
+    def luminosities_latent(self) -> np.ndarray:
         """
         The true luminosities of the objects. These are always latent
         as one cannot directly observe them
@@ -226,14 +245,14 @@ class Population(object):
         return self._luminosities
 
     @property
-    def distances(self) -> NDArray[np.float64]:
+    def distances(self) -> np.ndarray:
         """
         The distances to the objects
         """
         return self._distances
 
     @property
-    def known_distances(self) -> NDArray[np.float64]:
+    def known_distances(self) -> np.ndarray:
         """
         The observed distances
         """
@@ -241,21 +260,21 @@ class Population(object):
         return self._known_distances
 
     @property
-    def selection(self) -> NDArray[bool]:
+    def selection(self) -> np.ndarray:
         """
         The selection vector
         """
         return self._selection
 
     @property
-    def fluxes_latent(self) -> NDArray[np.float64]:
+    def fluxes_latent(self) -> np.ndarray:
         """
         The latent fluxes of the objects
         """
         return self._fluxes
 
     @property
-    def fluxes_observed(self) -> NDArray[np.float64]:
+    def fluxes_observed(self) -> np.ndarray:
         """
         All of the observed fluxes, i.e.,
         scattered with error
@@ -264,7 +283,7 @@ class Population(object):
         return self._flux_obs
 
     @property
-    def selected_fluxes_observed(self) -> NDArray[np.float64]:
+    def selected_fluxes_observed(self) -> np.ndarray:
         """
         The selected obs fluxes
         """
@@ -272,7 +291,7 @@ class Population(object):
         return self._flux_selected
 
     @property
-    def selected_fluxes_latent(self) -> NDArray[np.float64]:
+    def selected_fluxes_latent(self) -> np.ndarray:
         """
         The selected latent fluxes
         """
@@ -280,7 +299,7 @@ class Population(object):
         return self._fluxes[self._selection]
 
     @property
-    def selected_distances(self) -> NDArray[np.float64]:
+    def selected_distances(self) -> np.ndarray:
         """
         The selected distances. Note, this is different than
         the KNOWN distances
@@ -288,7 +307,7 @@ class Population(object):
         return self._distance_selected
 
     @property
-    def hidden_fluxes_observed(self) -> NDArray[np.float64]:
+    def hidden_fluxes_observed(self) -> np.ndarray:
         """
         The observed fluxes that are hidden by the selection
         """
@@ -296,7 +315,7 @@ class Population(object):
         return self._flux_hidden
 
     @property
-    def hidden_distances(self) -> NDArray[np.float64]:
+    def hidden_distances(self) -> np.ndarray:
         """
         The distances that are hidden by the selection
         """
@@ -304,7 +323,7 @@ class Population(object):
         return self._distance_hidden
 
     @property
-    def hidden_fluxes_latent(self) -> NDArray[np.float64]:
+    def hidden_fluxes_latent(self) -> np.ndarray:
         """
         The latent fluxes that are hidden by the selection
         """
@@ -372,10 +391,15 @@ class Population(object):
             known_z_obs=self._known_distances,
             z_idx=self._known_distance_idx + 1,  # stan indexing
             z_nidx=self._unknown_distance_idx + 1,  # stan indexing
+            r_obs=self._distance_selected,
+            known_r_obs=self._known_distances,
+            r_idx=self._known_distance_idx + 1,  # stan indexing
+            r_nidx=self._unknown_distance_idx + 1,  # stan indexing
             log_flux_obs=np.log10(self._flux_selected),
             flux_obs=self._flux_selected,
             flux_sigma=self._flux_sigma,
             z_max=self._r_max,
+            r_max=self._r_max,
             N_model=self._n_model,
             boundary=self._boundary,
             strength=self._strength,
@@ -386,7 +410,7 @@ class Population(object):
 
             output[k] = v
 
-        for k, v in self._auxiliary_quantites.items():
+        for k, v in self._auxiliary_quantities.items():
 
             output["%s_obs" % k] = v["obs_values"][self._selection]
 
@@ -396,7 +420,6 @@ class Population(object):
 
                 output[k] = int(v)
 
-            
         return output
 
     def writeto(self, file_name: str) -> None:
@@ -415,7 +438,9 @@ class Population(object):
 
             for k, v in self._spatial_params.items():
 
-                spatial_grp.create_dataset(k, data=np.array([v]), compression="lzf")
+                spatial_grp.create_dataset(k,
+                                           data=np.array([v]),
+                                           compression="lzf")
 
             if self._lf_params is not None:
 
@@ -423,7 +448,9 @@ class Population(object):
 
                 for k, v in self._lf_params.items():
 
-                    lum_grp.create_dataset(k, data=np.array([v]), compression="lzf")
+                    lum_grp.create_dataset(k,
+                                           data=np.array([v]),
+                                           compression="lzf")
 
                 f.attrs["lf_form"] = np.string_(self._lf_form)
 
@@ -443,36 +470,44 @@ class Population(object):
             f.attrs["distance_probability"] = self._distance_probability
             f.attrs["hard_cut"] = self._hard_cut
 
-            f.create_dataset("luminosities", data=self._luminosities, compression="lzf")
-            f.create_dataset("distances", data=self._distances, compression="lzf")
-            f.create_dataset(
-                "known_distances", data=self._known_distances, compression="lzf"
-            )
-            f.create_dataset(
-                "known_distance_idx", data=self._known_distance_idx, compression="lzf"
-            )
+            f.create_dataset("luminosities",
+                             data=self._luminosities,
+                             compression="lzf")
+            f.create_dataset("distances",
+                             data=self._distances,
+                             compression="lzf")
+            f.create_dataset("known_distances",
+                             data=self._known_distances,
+                             compression="lzf")
+            f.create_dataset("known_distance_idx",
+                             data=self._known_distance_idx,
+                             compression="lzf")
             f.create_dataset(
                 "unknown_distance_idx",
                 data=self._unknown_distance_idx,
                 compression="lzf",
             )
             f.create_dataset("fluxes", data=self._fluxes, compression="lzf")
-            f.create_dataset("flux_obs", data=self._flux_obs, compression="lzf")
-            f.create_dataset("selection", data=self._selection, compression="lzf")
+            f.create_dataset("flux_obs",
+                             data=self._flux_obs,
+                             compression="lzf")
+            f.create_dataset("selection",
+                             data=self._selection,
+                             compression="lzf")
             f.create_dataset("theta", data=self._theta, compression="lzf")
             f.create_dataset("phi", data=self._phi, compression="lzf")
 
             aux_grp = f.create_group("auxiliary_quantities")
 
-            for k, v in self._auxiliary_quantites.items():
+            for k, v in self._auxiliary_quantities.items():
 
                 q_grp = aux_grp.create_group(k)
-                q_grp.create_dataset(
-                    "true_values", data=v["true_values"], compression="lzf"
-                )
-                q_grp.create_dataset(
-                    "obs_values", data=v["obs_values"], compression="lzf"
-                )
+                q_grp.create_dataset("true_values",
+                                     data=v["true_values"],
+                                     compression="lzf")
+                q_grp.create_dataset("obs_values",
+                                     data=v["obs_values"],
+                                     compression="lzf")
 
             model_grp = f.create_group("model_spaces")
 
@@ -484,8 +519,7 @@ class Population(object):
             recursively_save_dict_contents_to_group(f, "truth", self._truth)
 
             recursively_save_dict_contents_to_group(
-                f, "graph", fill_graph_dict(nx.to_dict_of_dicts(self._graph))
-            )
+                f, "graph", fill_graph_dict(nx.to_dict_of_dicts(self._graph)))
 
     @classmethod
     def from_file(cls, file_name):
@@ -545,7 +579,8 @@ class Population(object):
             try:
                 known_distances = f["known_distances"][()]
                 known_distance_idx = (f["known_distance_idx"][()]).astype(int)
-                unknown_distance_idx = (f["unknown_distance_idx"][()]).astype(int)
+                unknown_distance_idx = (
+                    f["unknown_distance_idx"][()]).astype(int)
 
             except:
 
@@ -568,15 +603,17 @@ class Population(object):
             for k in f["auxiliary_quantities"].keys():
 
                 auxiliary_quantities[str(k)] = {
-                    "true_values": f["auxiliary_quantities"][k]["true_values"][()],
-                    "obs_values": f["auxiliary_quantities"][k]["obs_values"][()],
+                    "true_values":
+                    f["auxiliary_quantities"][k]["true_values"][()],
+                    "obs_values":
+                    f["auxiliary_quantities"][k]["obs_values"][()],
                 }
 
             truth = recursively_load_dict_contents_from_group(f, "truth")
 
             graph = nx.from_dict_of_dicts(
-                clean_graph_dict(recursively_load_dict_contents_from_group(f, "graph"))
-            )
+                clean_graph_dict(
+                    recursively_load_dict_contents_from_group(f, "graph")))
 
         return cls(
             luminosities=luminosities,
@@ -608,6 +645,87 @@ class Population(object):
             phi=phi,
         )
 
+    def to_sub_population(self, observed: bool = True) -> "Population":
+        """
+        Create a population that is down selected from either the 
+        observed or unobserved population
+
+        :param observed: extract the observed or unobserved object
+
+        """
+
+        if observed:
+            selection = self._selection
+
+        else:
+
+            selection = ~self._selection
+
+        if self._auxiliary_quantities is not None:
+
+            new_aux = {}
+
+            for k, v in self._auxiliary_quantities.items():
+
+                new_aux[k] = {
+                    "true_values": v["true_values"][selection],
+                    "obs_values": v["obs_values"][selection]
+                }
+
+        else:
+
+            new_aux = None
+
+        itr = 0
+        known_distances = []
+        known_distance_idx = []
+        unknown_distance_idx = []
+
+        for i, s in enumerate(selection):
+
+            if s:
+
+                if i in self._known_distances:
+
+                    known_distances.append(self._distances[i])
+                    known_distance_idx.append(itr)
+
+                else:
+
+                    unknown_distance_idx.append(itr)
+
+                itr += 1
+
+        return Population(
+            luminosities=self._luminosities[selection],
+            distances=self._distances[selection],
+            known_distances=np.array(known_distances),
+            known_distance_idx=np.array(known_distance_idx),
+            unknown_distance_idx=np.array(unknown_distance_idx),
+            fluxes=self._fluxes[selection],
+            flux_obs=self._flux_obs[selection],
+            selection=np.ones(sum(selection), dtype=bool),
+            flux_sigma=self._flux_sigma,
+            n_model=self._n_model,
+            r_max=self._r_max,
+            boundary=self._boundary,
+            strength=self._strength,
+            lf_params=self._lf_params,
+            spatial_params=self._spatial_params,
+            model_spaces=self._model_spaces,
+            seed=self._seed,
+            name=self._name,
+            spatial_form=self._spatial_form,
+            lf_form=self._lf_form,
+            auxiliary_quantities=new_aux,
+            truth=self._truth,
+            distance_probability=self._distance_probability,
+            hard_cut=self._hard_cut,
+            graph=self._graph,
+            theta=self._theta[selection],
+            phi=self._phi[selection],
+        )
+
     def display(self):
         """
         Display the simulation parameters
@@ -680,7 +798,9 @@ class Population(object):
 
         try:
 
-            ax.set_ylim(bottom=min([self._fluxes.min(), self._flux_selected.min()]))
+            ax.set_ylim(
+                bottom=min([self._fluxes.min(),
+                            self._flux_selected.min()]))
 
         except:
 
@@ -703,7 +823,7 @@ class Population(object):
         # to plot
 
         if self._no_detection:
-            print("There are no detections to display")
+            log.warning("There are no detections to display")
             if ax is not None:
 
                 fig = ax.get_figure()
@@ -734,7 +854,8 @@ class Population(object):
         # ax.set_xscale('log')
         ax.set_yscale("log")
 
-        ax.set_ylim(bottom=min([self._fluxes.min(), self._flux_selected.min()]))
+        ax.set_ylim(bottom=min([self._fluxes.min(),
+                                self._flux_selected.min()]))
         ax.set_xlim(right=self._r_max)
 
         ax.set_xlabel("distance")
@@ -776,9 +897,9 @@ class Population(object):
 
         if (with_arrows) and (not self._no_detection):
             for start, stop, z in zip(
-                self._fluxes[self._selection],
-                self._flux_selected,
-                self._distance_selected,
+                    self._fluxes[self._selection],
+                    self._flux_selected,
+                    self._distance_selected,
             ):
 
                 x = z
@@ -824,12 +945,14 @@ class Population(object):
 
             fig = ax.get_figure()
 
-        ax.scatter(
-            self._distance_selected, self._luminosity_selected, s=5, color=obs_color
-        )
-        ax.scatter(
-            self._distance_hidden, self._luminosity_hidden, s=5, color=true_color
-        )
+        ax.scatter(self._distance_selected,
+                   self._luminosity_selected,
+                   s=5,
+                   color=obs_color)
+        ax.scatter(self._distance_hidden,
+                   self._luminosity_hidden,
+                   s=5,
+                   color=true_color)
 
         return fig
 
@@ -849,7 +972,7 @@ class Population(object):
     ):
 
         if len(fluxes) == 0:
-            print("There are no detections to display")
+            log.warning("There are no detections to display")
 
             return
 
