@@ -1,7 +1,7 @@
 import numpy as np
 
 
-def sample_theta_phi(size):
+def sample_theta_phi(size: int):
 
     theta = np.arccos(1 - 2 * np.random.uniform(0.0, 1.0, size=size))
     phi = np.random.uniform(0, 2 * np.pi, size=size)
